﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
//using System.Threading.Tasks;
using System.Windows.Forms;

namespace DistanceSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		DistanceSensor openCh = null;
		DistanceSensor distance = null;
		private ErrorEventBox errorBox;
		formCleaner cleaner;

		private DataGraphBox distanceGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(DistanceSensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			phidgetInfoBox1.Clear();


			errorBox = new ErrorEventBox(this);
			distance = openCh ?? openArgs.makeChannel<DistanceSensor>();

			distanceGraphBox = new DataGraphBox(this, "Distance");

			distance.Attach += distance_attach;
			distance.Detach += distance_detach;
			distance.Error += distance_error;
			distance.SonarReflectionsUpdate += distance_update;
			distance.DistanceChange += distance_change;
			distance.PropertyChange += distance_propertychange;

			try {
				distance.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			distance.Attach -= distance_attach;
			distance.Detach -= distance_detach;
			distance.Error -= distance_error;
			distance.SonarReflectionsUpdate -= distance_update;
			distance.DistanceChange -= distance_change;
			distance.PropertyChange -= distance_propertychange;
			distance.Close();
		}

		void distance_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			DistanceSensor attachedDevice = (DistanceSensor)sender;

			distanceGraphBox.SetTitle(DataGraphBox.GenerateTitle("Distance", (Phidget)distance));

			try {//configure form controls
				dataIntervalTrk.Unit = " ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				//differing max/min means the interval is a settable configuration
				if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
					dataIntervalTrk.Enabled = false;



				changeTriggerTrk.Unit = "mm";
				changeTriggerTrk.minTxt = attachedDevice.MinDistanceChangeTrigger.ToString();
				changeTriggerTrk.maxTxt = attachedDevice.MaxDistanceChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinDistanceChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxDistanceChangeTrigger;
				changeTriggerTrk.Value = attachedDevice.DistanceChangeTrigger;

				if (attachedDevice.DeviceID == DeviceID.PN_DST1200) {
					attachedDevice.SonarQuietMode = true;
					sonarQuietModeBox.Visible = true;
					sonarQuietModeBox.Checked = true;
					sonarReflectionsBox.Visible = true;
				}
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting up sonar sensor: " + ex.Message);
			}

			settingsBox.Visible = true;
			dataBox.Visible = true;

		}
		void distance_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			sonarQuietModeBox.Visible = false;
			sonarReflectionsBox.Visible = false;
			dataBox.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}
		void distance_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation || e.Code == ErrorEventCode.OutOfRange)
				distanceGraphBox.addGap();
		}

		private void distance_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = distance.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "DistanceChangeTrigger":
				changeTriggerTrk.Value = distance.DistanceChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = distance.DataRate;
					updateGraphDataInterval();
				}
				break;
			}
		}
		void distance_change(object sender, Phidget22.Events.DistanceSensorDistanceChangeEventArgs e) {
			try {
				valueTxt.Text = e.Distance.ToString() + "mm";
				distanceGraphBox.addData(e.Distance, distance.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage(ex.ToString()); }
		}

		void distance_update(object sender, Phidget22.Events.DistanceSensorSonarReflectionsUpdateEventArgs e) {
			try {
				countTxt.Text = e.Count.ToString();

				distance0.Text = (0 < e.Count) ? e.Distances[0].ToString() : "";
				distance1.Text = (1 < e.Count) ? e.Distances[1].ToString() : "";
				distance2.Text = (2 < e.Count) ? e.Distances[2].ToString() : "";
				distance3.Text = (3 < e.Count) ? e.Distances[3].ToString() : "";
				distance4.Text = (4 < e.Count) ? e.Distances[4].ToString() : "";
				distance5.Text = (5 < e.Count) ? e.Distances[5].ToString() : "";
				distance6.Text = (6 < e.Count) ? e.Distances[6].ToString() : "";
				distance7.Text = (7 < e.Count) ? e.Distances[7].ToString() : "";

				amplitude0.Text = (0 < e.Count) ? e.Amplitudes[0].ToString() : "";
				amplitude1.Text = (1 < e.Count) ? e.Amplitudes[1].ToString() : "";
				amplitude2.Text = (2 < e.Count) ? e.Amplitudes[2].ToString() : "";
				amplitude3.Text = (3 < e.Count) ? e.Amplitudes[3].ToString() : "";
				amplitude4.Text = (4 < e.Count) ? e.Amplitudes[4].ToString() : "";
				amplitude5.Text = (5 < e.Count) ? e.Amplitudes[5].ToString() : "";
				amplitude6.Text = (6 < e.Count) ? e.Amplitudes[6].ToString() : "";
				amplitude7.Text = (7 < e.Count) ? e.Amplitudes[7].ToString() : "";
			} catch (PhidgetException ex) { errorBox.addMessage(ex.ToString()); }
		}

		private void sonarQuietModeBox_CheckedChanged(object sender, EventArgs e) {
			try {
				distance.SonarQuietMode = sonarQuietModeBox.Checked;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting sonar quiet mode: " + ex.Message);
			}
		}

		private void updateGraphDataInterval() {
			try {
				if (distance.DistanceChangeTrigger == 0)
					distanceGraphBox.DataInterval = (1000.0 / distance.DataRate);
				else
					distanceGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

		private void distanceGraphButton_Click(object sender, EventArgs e) {
			if (!distanceGraphBox.Visible)
				distanceGraphBox.Show();
		}

		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temp = dataIntervalTrk.Value;
						distance.DataInterval = (int)temp;
						if (distance.DataInterval < (int)temp) {
							while (distance.DataInterval != (int)temp) {
								temp++;
								distance.DataInterval = (int)temp;
							}
						} else {
							while (distance.DataInterval != (int)temp) {
								temp--;
								distance.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = distance.DataInterval;
						dataIntervalTrk.Value = distance.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						distance.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			} else //changetrigger control
			  {
				try {
					distance.DistanceChangeTrigger = (int)changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = distance.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = distance.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (distance.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = distance.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = distance.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}